@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2013	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -34,6 +34,7 @@
 #include "libpq-fe.h"
 #include <stdio.h>
 #include <time.h>
+#include <sys/time.h>
 #include <sys/types.h>
 #include <limits.h>
 
@@ -78,7 +79,6 @@ typedef enum {
 	POOL_CONTINUE = 0,
 	POOL_IDLE,
 	POOL_END,
-	POOL_END_WITH_FRONTEND_ERROR,
 	POOL_ERROR,
 	POOL_FATAL,
 	POOL_DEADLOCK
@@ -164,6 +164,9 @@ typedef struct {
 	char *buf2;	/* buffer for pool_read2 */
 	int bufsz2;	/* its size in bytes */
 
+	char *buf3;	/* buffer for pool_push/pop */
+	int bufsz3;	/* its size in bytes */
+
 	int isbackend;		/* this connection is for backend if non 0 */
 	int db_node_id;		/* DB node id for this connection */
 
@@ -276,6 +279,13 @@ extern int my_master_node_id;
 	 ((*(my_backend_status[(backend_id)]) == CON_UP) ||			\
 	  (*(my_backend_status[(backend_id)]) == CON_CONNECT_WAIT))))
 
+/*
+ * For raw mode failover control
+ */
+#define VALID_BACKEND_RAW(backend_id) \
+	((*(my_backend_status[(backend_id)]) == CON_UP) ||			\
+	 (*(my_backend_status[(backend_id)]) == CON_CONNECT_WAIT))
+
 #define CONNECTION_SLOT(p, slot) ((p)->slots[(slot)])
 #define CONNECTION(p, slot) (CONNECTION_SLOT(p, slot)->con)
 
@@ -311,12 +321,10 @@ extern int my_master_node_id;
 
 #define REPLICATION (pool_config->replication_mode)
 #define MASTER_SLAVE (pool_config->master_slave_mode)
-#define STREAM (MASTER_SLAVE && !strcmp("stream", pool_config->master_slave_sub_mode))
-#define SLONY (MASTER_SLAVE && !strcmp("slony", pool_config->master_slave_sub_mode))
 #define DUAL_MODE (REPLICATION || MASTER_SLAVE)
 #define PARALLEL_MODE (pool_config->parallel_mode)
 #define RAW_MODE (!REPLICATION && !PARALLEL_MODE && !MASTER_SLAVE)
-#define MAJOR(p) (pool_get_major_version())
+#define MAJOR(p) MASTER_CONNECTION(p)->sp->major
 #define TSTATE(p, i) (CONNECTION(p, i)->tstate)
 #define INTERNAL_TRANSACTION_STARTED(p, i) (CONNECTION(p, i)->is_internal_transaction_started)
 #define SYSDB_INFO (system_db_info->info)
@@ -333,9 +341,11 @@ extern int my_master_node_id;
 #define NO_LOAD_BALANCE "/*NO LOAD BALANCE*/"
 #define NO_LOAD_BALANCE_COMMENT_SZ (sizeof(NO_LOAD_BALANCE)-1)
 
-#define MAX_NUM_SEMAPHORES		3
+#define MAX_NUM_SEMAPHORES		4
 #define CONN_COUNTER_SEM 0
 #define REQUEST_INFO_SEM 1
+#define SHM_CACHE_SEM	2
+#define QUERY_CACHE_STATS_SEM	3
 
 /*
  * number specified when semaphore is locked/unlocked
@@ -389,6 +399,16 @@ typedef struct {
 	char **data;		/* actual row character data terminated with null */
 } POOL_SELECT_RESULT;
 
+/*
+ * recovery mode
+ */
+typedef enum {
+	RECOVERY_INIT = 0,
+	RECOVERY_ONLINE,
+	RECOVERY_DETACH,
+	RECOVERY_PROMOTE
+} POOL_RECOVERY_MODE;
+
 /*
  * global variables
  */
@@ -397,6 +417,7 @@ extern bool run_as_pcp_child;
 
 extern POOL_CONNECTION_POOL *pool_connection_pool;	/* connection pool */
 extern volatile sig_atomic_t backend_timer_expired; /* flag for connection closed timer is expired */
+extern volatile sig_atomic_t health_check_timer_expired;		/* non 0 if health check timer expired */
 extern long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
 extern int my_proc_id;  /* process table id (!= UNIX's PID) */
 extern POOL_SYSTEMDB_CONNECTION_POOL *system_db_info; /* systemdb */
@@ -413,6 +434,9 @@ extern char query_string_buffer[];		/* last query string sent to simpleQuery() *
 
 extern BACKEND_STATUS private_backend_status[MAX_NUM_BACKENDS];
 
+extern char remote_host[];	/* client host */
+extern char remote_port[];	/* client port */
+
 /*
  * public functions
  */
@@ -457,7 +481,6 @@ extern POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend,
 
 extern void notice_backend_error(int node_id);
 extern void degenerate_backend_set(int *node_id_set, int count);
-extern bool degenerate_backend_set_ex(int *node_id_set, int count, bool test_only);
 extern void promote_backend(int node_id);
 extern void send_failback_request(int node_id);
 
@@ -510,9 +533,6 @@ extern int send_startup_packet(POOL_CONNECTION_POOL_SLOT *cp);
 extern void pool_free_startup_packet(StartupPacket *sp);
 extern void child_exit(int code);
 
-extern int health_check(void);
-extern int system_db_health_check(void);
-
 extern void init_prepared_list(void);
 
 extern void *pool_shared_memory_create(size_t size);
@@ -531,7 +551,7 @@ extern POOL_STATUS OneNode_do_command(POOL_CONNECTION *frontend, POOL_CONNECTION
 
 /* child.c */
 extern POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
-	char *hostname, int port, char *dbname, char *user, char *password);
+	char *hostname, int port, char *dbname, char *user, char *password, bool retry);
 extern void discard_persistent_db_connection(POOL_CONNECTION_POOL_SLOT *cp);
 
 /* define pool_system.c */
@@ -567,6 +587,7 @@ extern void init_ps_display(const char *username, const char *dbname,
 							const char *host_info, const char *initial_str);
 extern void set_ps_display(const char *activity, bool force);
 extern const char *get_ps_display(int *displen);
+extern void pool_ps_idle_display(POOL_CONNECTION_POOL *backend);
 
 /* recovery.c */
 extern int start_recovery(int recovery_node);
